#!/usr/bin/perl

# copyfits.pl

# Usage copyfits.pl
# Reads all files in asciifile FilesToCopy.txt to fix invalid fitsheaders and stores it 
# in a file named cleansauberN.fits. N is a number.

use strict;
use warnings;
use File::Copy ;

our $in_file;
our $in_file_size;
our $file_extension;
our $out_file_root = "headersauber";
our $out_file;
our $old_header;
our $new_header = "";
our $data;
our $number_of_headerlines;
our $header_size;
our $headerline;
our $dateline;
our $datei;
our ($i,$offset);
our ($dateobs, $time, $h, $m, $s, $tmsec, $tmstart, $before, $after);
our $timeobs;
our $spaces_needed = 0;
our $spaces = "";
our $nullbytes_needed = 0;
our $nullbytes = "";
our $num = 0;


# clean current directory

unlink glob $out_file_root."[0-9]*.fits";

open( FTC, "FilesToCopy.txt" );
while ( <FTC> ) {
    chomp;
    $in_file = $_;
    $out_file = $out_file_root.$num++.".fits";
    $new_header = "";
    $spaces = "";

    if ($ARGV[0] =~ /[yY]/){
      open( F, $in_file );
      $in_file_size = (stat($in_file))[7];
      read F, $datei, $in_file_size, 0;
  
      $datei =~ /SIMPLE.*END\ */;
      $old_header = $&;
      $data = $'; #' this is only for better syntax highlighting in emacs ;-)
     if ( substr($old_header,0,6) ne "SIMPLE" ){ 
     	print $in_file." is not a Fits file. -> Skipped!\n";
     	exit;
     }
  
     # MaximDL:
     if ( index($old_header,"TIME-OBS") != -1 ) { 
        $timeobs = substr($old_header, index($old_header, "TIME-OBS"), 80); 
        ($before,$time,$after) = split(/'/,$timeobs);
     } 
     
     $number_of_headerlines = length($old_header) / 80;
     for ( $i=0; $i < $number_of_headerlines ; $i++ ){
         $offset = ( $i * 80 );
         $headerline = substr($old_header, $offset, 80);
         if ( index( $old_header, "EXPTIME" ) == -1 ) {
            $headerline =~ s/EXPOSURE/EXPTIME /;                                       # Michna
         }
         $headerline =~ s/e\+/E\+/;                                                    # CCDSoft
         $headerline =~ s/(DATE-OBS= '[0-9]{4})\/([0-9]{2})\/([0-9]{2}T)/$1-$2-$3/;    # Michna
         $headerline =~ s/(DATE    = '[0-9]{4})\/([0-9]{2})\/([0-9]{2}T)/$1-$2-$3/;    # Michna
         $headerline =~ s/(DATE    = ')([0-9]{2}).([0-9]{2}).([0-9]{4})'/$1$4-$3-$2'/; # pisco
         $headerline =~ s/Standorth.he\ .\.n\.N\ \ /Standorthoehe\ ue\.n\.N/;          # Michna
         $headerline =~ s/\ {80}//;
         $new_header .= $headerline;
     
         if ( $headerline =~ m/^DATE-OBS/) {
     	    $dateline = $headerline;
     	    $dateline =~ s/DATE-OBS/CIV-DATE/; #create a header CIVIL-DATE
     	    $new_header .= $dateline;
     	    if ( index( $old_header, "TM-START" ) == -1 ) {
     		($before,$dateobs,$after) = split(/'/,$headerline);
     		if ( $dateobs =~ m/T/ ){
     		   ($before,$time) = split(/T/,$dateobs);
     		}
     		($h,$m,$s) = split(/:/,$time);
     		$tmsec = $h*3600 + $m*60 + $s;
     		$tmstart = "TM-START=        ".$tmsec."     / [s] UT of start in sec after midnight                                                      ";
     		$new_header .= substr($tmstart, 0, 80);
     	    }
         }
         if ( $i == 7 && index( $old_header, "CRPIX" ) == -1 ){
     	    $new_header .= "CRPIX1  =                   1. / Reference pixel                                ";
     	    $new_header .= "CRPIX2  =                   1. / Reference pixel                                ";
         }
         if ( $i == 7 && index( $old_header, "CRVAL" ) == -1 ){
                 $new_header .= "CRVAL1  =                   1. / Coordinate at reference pixel                  ";
     	    $new_header .= "CRVAL2  =                   1. / Coordinate at reference pixel                  ";
         }
         if ( $i == 7 && index( $old_header, "CDELT" ) == -1 ){
     	    $new_header .= "CDELT1  =                   1. / Coord. incr. per pixel (original value)        ";
     	    $new_header .= "CDELT2  =                   1. / Coord. incr. per pixel (original value)        ";
         }
         if ( $i == 7 && index( $old_header, "CTYPE" ) == -1 ){
                 $new_header .= "CTYPE1  = '                '   / Units of coordinate                            ";
     	    $new_header .= "CTYPE2  = '                '   / Units of coordinate                            ";
     	}
         if ( $i == 7 && index( $old_header, "BUNIT" ) == -1 ){
     	    $new_header .= "BUNIT   = '                '   / Units of data values                           ";
         }
     }
     
     if ( length($new_header) % 2880 != 0 ){
         $spaces_needed = 2880 - length($new_header) % 2880;
     }
     
     for ( $i=0; $i < $spaces_needed; $i++ ) { $spaces .= " " }
     $new_header .= $spaces;
     
     
     $data =~ s/\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0//g;
     
     if ( length($data) % 2880 != 0 ){
         $nullbytes_needed = 2880 - length($data) % 2880;
     }
     
     for ( $i=0; $i < $nullbytes_needed; $i++ ) { $nullbytes .= "\0" }
     $data .= $nullbytes;
     
     open( W, "> $out_file" );
     print W $new_header.$data;
  }
  else
  {copy $in_file, $out_file;}
}

